<?php

namespace Api\PhpServices\Session;

use Api\PhpUtils\Http\Request;
use Api\PhpUtils\Redis\RedisUtil;

/**
 * Customer Redis Session Handler
 */
class RedisSessionHandler implements \SessionHandlerInterface
{
    protected $key = 'c071f6e5923a648a5ef88f32b6f738e1';
    protected $expire = 0;
    protected $from = 'bpsession';
    protected $redis = null;

    /**
     * open session
     * @param string $savePath
     * @param string $sessionName
     * @return boolean
     */
    public function open($savePath, $sessionName)
    {
        $this->getRedis();
        return true;
    }

    /**
     * read session
     * @param string $sessionId
     * @return string
     */
    public function read($sessionId)
    {
        $data = $this->redis->get($sessionId);
        $GLOBALS['SESSION_CONNECT'] = $this->redis->getConnectStatus();
        return empty($data) ? '' : (is_array($data) ? serialize($data) : $data);
    }

    /**
     * write session
     * @param string $sessionId
     * @param string $sessionData
     * @return boolean
     */
    public function write($sessionId, $sessionData)
    {
        if (!$_SESSION) {
            return true;
        }
        $this->getRedis();
        $retry = 3;
        while ($retry--) {
            $result = $this->redis->setEx($sessionId,$this->expire,$_SESSION);
            if ($result !== false) {
                return true;
            }
        }
        return false;
    }

    /**
     * close session
     * @return boolean
     */
    public function close()
    {
        return true;
    }

    /**
     * destroy session
     * @param string $sessionId
     * @return boolean
     */
    public function destroy($sessionId)
    {
        $this->getRedis();
        return $this->redis->del($sessionId);
    }

    /**
     * garbage clean session
     * session in redis would be auto cleaned when expired
     * @param integer $maxlifetime
     * @return integer
     */
    public function gc($maxlifetime)
    {
        return 0;
    }

    /**
     * set session expire
     * @param integer $expire   second
     * @return RedisSessionHandler
     */
    public function setExpire($expire)
    {
        $this->expire = $expire;
        return $this;
    }

    /**
     * get session expire
     * @return integer
     */
    public function getExpire()
    {
        return $this->expire;
    }

    /**
     * get redis instance
     * @return RedisUtil
     */
    public function getRedis()
    {
        if (is_null($this->redis)) {
            $this->redis = RedisUtil::getInstance('redis', ['serverRegion'=> 'master', 'serializer' => 'php', 'master'=> true]);
        }
        return $this->redis;
    }
}
